/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 19/05/2007
   derniere modification : 18/06/2007 */


#ifndef __SCEMATERIAL_H__
#define __SCEMATERIAL_H__


/* constantes generales */
/* types de maps */
#define SCE_COLOR_MAP 0
#define SCE_NORMAL_MAP 1
#define SCE_HEIGHT_MAP 2
#define SCE_EMISSIVE_MAP 3
#define SCE_GLOSS_MAP 4
#define SCE_CUBE_MAP 5
/* nombre de types de maps */
#define SCE_NUM_MAPS 6
/* represente toutes les maps */
#define SCE_ALL_MAPS -1

/* unites de texture des maps */
#define SCE_TEXUNIT0 0
#define SCE_TEXUNIT1 1
#define SCE_TEXUNIT2 2
#define SCE_TEXUNIT3 3
#define SCE_TEXUNIT4 4
#define SCE_TEXUNIT5 5
#define SCE_TEXUNIT6 6
#define SCE_TEXUNIT7 7


/* identifiants des donnes d'une face d'une cubemap */
#define SCE_TEXID_POSX 0
#define SCE_TEXID_NEGX 1
#define SCE_TEXID_POSY 2
#define SCE_TEXID_NEGY 3
#define SCE_TEXID_POSZ 4
#define SCE_TEXID_NEGZ 5



/* structure d'une texture pour un materiau */
typedef struct sce_smattex SCE_SMatTex;
struct sce_smattex
{
    SCE_STexture *tex;      /* pointeur sur la structure d'une texture */
    int actived;            /* booleen, true: texture activee */
    int user;               /* booleen, true: envoyee par l'utilisateur */
};

#if 0
/* structure d'un shader pour un materiau */
typedef struct sce_smatshd SCE_SMatShd;
struct sce_smatshd
{
    SCE_SShader *shd;       /* pointeur sur la structure d'une texture */
    int actived;            /* booleen, true: shader active */
};
#endif


/* structure d'un materiau */
typedef struct sce_smaterial SCE_SMaterial;
struct sce_smaterial
{
    /* materiau du coeur */
    SCE_CMaterial *cmat;
    
    /* tableau des textures, classees par indices (voir "types de maps"),
       et par unites de texture */
    SCE_SMatTex *maps[SCE_NUM_MAPS];
    
    /* shader du materiau */
    SCE_SShader *shader;
    
    /* defini si le shader est actif pour le rendu */
    int shader_actived;
    
    /* defini si le shader appartient a l'utilisateur ou non */
    int shader_user;
};


/* cree un materiau */
SCE_SMaterial* SCE_Material_Create(void);

/* supprime un materiau */
void SCE_Material_Delete(SCE_SMaterial*);


/* defini une couleur d'un materiau */
void SCE_Material_SetColor(SCE_SMaterial*, SCEenum, float, float, float, float);
void SCE_Material_SetColorv(SCE_SMaterial*, SCEenum, float*);

/* renvoie la premiere unite de texture libre pour un type donne */
int SCE_Material_GenTextureUnit(SCE_SMaterial*, int);

/* ajoute une texture a un materiau */
SCEuint SCE_Material_AddTexture_(SCE_SMaterial*, SCEenum, SCEuint,
                                 SCE_STexture*, int);
/* macro pour l'utilisateur utilisant la fonction, qui specifiera alors
   automatiquement que la texture est envoyee par l'utilisateur */
#define SCE_Material_AddTexture(a, b, c, d)\
    SCE_Material_AddTexture_(a, b, c, d, SCE_TRUE)

/* assigne un shader a un materiau */
void SCE_Material_SetShader_(SCE_SMaterial*, SCE_SShader*, int);
/* macro pour l'utilisateur utilisant la fonction, qui specifiera alors
   automatiquement que le shader est envoye par l'utilisateur */
#define SCE_Material_SetShader(a, b) SCE_Material_SetShader_(a, b, SCE_TRUE)



/* charge une texture pour le materiau */
SCEuint SCE_Material_LoadTexture(SCE_SMaterial*, SCEenum, SCEuint,
                                 const char*, int);
/* cube map */
SCEuint SCE_Material_LoadCubemap(SCE_SMaterial*, SCEuint,
                                const char*, const char*, const char*,
                                const char*, const char*, const char*, int);

/* charge un shader pour le materiau */
int SCE_Material_LoadShader(SCE_SMaterial*, const char*, const char*);


/* supprime une texture */
void SCE_Material_DeleteTexture(SCE_SMaterial*, int, int);

/* supprime le shader */
void SCE_Material_DeleteShader(SCE_SMaterial*);


/* active/desactive une texture */
void SCE_Material_ActiveTexture(SCE_SMaterial*, int, int, int);

/* active/desactive le shader */
void SCE_Material_ActiveShader(SCE_SMaterial*, int);

/* defini le materiau actif */
void SCE_Material_Use(SCE_SMaterial*);


#endif /* guard */
