/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 2 janvier 2007
   derniere modification le 06/06/2007 */


/*
    ResourceManager
    Interface, couche 2
*/


#include <stdio.h>

#include <SCE/SCEMinimal.h>

#include <SCE/utils/SCEString.h>

#include <SCE/interface/SCEMedia.h>

#include <SCE/interface/SCEResources.h>



typedef struct sce_sresource SCE_SResource;
struct sce_sresource
{
    /* donnees */
    void *data;
    
    /* nom */
    char *name;
    
    /* fonction de suppression de la ressource */
    /*void (*delete)(void*);*/
    
    /* nombre de fois utilisee */
    unsigned int nb_used;
};


/* nombre de ressources chargees */
static unsigned int nb_res;

/* ressources chargees */
static SCE_SResource *resources;

/* etat du gestionnaire */
static int sce_res_enabled;



int SCE_Resource_Init(void)
{
    static int is_init = 0;
    
    if(!is_init)
    {
        nb_res = 0;
        resources = NULL;
        is_init = 1;
    }
    
    return (SCE_OK);
}

int SCE_Resource_Quit(void)
{
    unsigned int i;
    
    for(i=0; i<nb_res; i++)
        SCE_free(resources[i].name);
    
    SCE_free(resources);
    resources = NULL;
    nb_res = 0;
    
    return (SCE_OK);
}


/* ajoute le 27/06/2007 */
void SCE_Resource_State(int state)
{
    sce_res_enabled = state;
}


/* recherche une ressource par son nom */
static int SCE_Resource_LocateName(const char *name)
{
    unsigned int i;
    
    for(i=0; i<nb_res; i++)
    {
        if(SCE_String_Cmp(name, resources[i].name, 1))
            return (i);
    }
    
    return (-1);
}

/* recherche une ressource par ses donnees */
static int SCE_Resource_LocateData(void *data)
{
    unsigned int i;
    
    for(i=0; i<nb_res; i++)
    {
        if(resources[i].data == data)
            return (i);
    }
    
    return (-1);
}


/* revise le 27/06/2007 */
void* SCE_Resource_Load(const char *filename, void *p1, void *p2)
{
    /* identifiant de la ressource a charger */
    int res_id = -1; /* initialisation importante */
    
    /* donnees a renvoyer */
    void *data = NULL;
    
    /* on commence par rechercher si la ressource a deja ete chargee */
    /* si le gestionnaire est desactive, res_id vaudra -1, donc le
       chargement d'une nouvelle ressource sera force */
    if(sce_res_enabled)
        res_id = SCE_Resource_LocateName(filename);
    
    if(res_id > -1)
    {
        /* ressource trouvee */
        
        data = resources[res_id].data;  /* recuperation des donnees */
        resources[res_id].nb_used++;    /* incrementation du compteur */
    }
    else
    {
        /* ressource non trouvee ou nouveau chargement force,
           creation d'une nouvelle ressource et chargement */
        
        data = SCE_Resource_LoadNew(filename, p1, p2);
        if(data == NULL)
            Logger_LogSrc();
    }
    
    /* on renvoie la ressource */
    return (data);
}

void* SCE_Resource_LoadNew(const char *filename, void *p1, void *p2)
{
    size_t slen;
    
    resources = SCE_realloc(resources, (nb_res+1) * sizeof *resources);
    if(resources == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    slen = strlen(filename) + 1;
    
    resources[nb_res].name = SCE_malloc(slen);
    if(resources[nb_res].name == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    memset(resources[nb_res].name, '\0', slen);
    strcpy(resources[nb_res].name, filename);
    
    resources[nb_res].nb_used = 1;
    
    /* chargement de la ressource via le MediaManager */
    resources[nb_res].data = SCE_Media_LoadFromFile(filename, p1, p2);
    if(resources[nb_res].data == NULL)
    {
        SCE_free(resources[nb_res].name);
        Logger_LogSrc();
        return (NULL);
    }
    
    /* +1 ressource */
    nb_res++;
    
    return (resources[nb_res-1].data);
}

int SCE_Resource_Delete(void *data)
{
    int id, ret = SCE_FALSE;
    
    id = SCE_Resource_LocateData(data);
    if(id != -1)
    {
        /* -1 utilisation */
        resources[id].nb_used--;
        
        if(resources[id].nb_used == 0)
        {
            /* la ressource n'est plus utilisee du tout, destruction */
            
            nb_res--;   /* -1 ressource */
            
            /* on supprime la memoire allouee pour la ressource */
            SCE_free(resources[id].name);
            
            /* on deplace les infos du dernier du tableau
               dans celui qu'on vient de supprimer */
            resources[id].name = resources[nb_res].name;
            resources[id].data = resources[nb_res].data;
            resources[id].nb_used = resources[nb_res].nb_used;
            
            /* on realloue l'espace afin de le reduire */
            resources = SCE_realloc(resources, nb_res * sizeof *resources);
            if(resources == NULL && nb_res > 0)
            {
                Logger_LogSrc();
                ret = SCE_ERROR;
            }
            else
            {
                /* on autorise la destruction du media par l'utilisateur */
                ret = SCE_TRUE;
            }
        }
    }
    #ifdef __SCE_DEBUG__ /* osef... */
    else
    {
        /* tentative de destruction d'une ressource inconnue */
        Logger_Log(SCE_INVALID_OPERATION);
        Logger_LogMsg("you can't destroy a non-loaded resource !");
        ret = SCE_ERROR;
    }
    #endif
    
    return (ret);
}


int SCE_Resource_NbUsed(const char *name, void *data)
{
    int id, final = -1;
    
    if(name != NULL)
        final = SCE_Resource_LocateName(name);
    
    if(data != NULL)
    {
        id = SCE_Resource_LocateData(data);
        if(final == -1)
            final = id;
        else if(final != id)
            return (-1);
    }
    
    return (resources[final].nb_used);
}

int SCE_Resource_NbLoaded(void)
{
    return (nb_res);
}

char* SCE_Resource_GetName(void *data)
{
    int id;
    
    id = SCE_Resource_LocateData(data);
    if(id > -1)
        return (resources[id].name);
    else
        return (NULL);
}
